# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import AssetCollectionModuleParamsClass
from .....schema_classes import DataHubPageModuleParamsClass
from .....schema_classes import DataHubPageModulePropertiesClass
from .....schema_classes import DataHubPageModuleTypeClass
from .....schema_classes import DataHubPageModuleVisibilityClass
from .....schema_classes import HierarchyModuleParamsClass
from .....schema_classes import LinkModuleParamsClass
from .....schema_classes import PageModuleScopeClass
from .....schema_classes import RichTextModuleParamsClass


AssetCollectionModuleParams = AssetCollectionModuleParamsClass
DataHubPageModuleParams = DataHubPageModuleParamsClass
DataHubPageModuleProperties = DataHubPageModulePropertiesClass
DataHubPageModuleType = DataHubPageModuleTypeClass
DataHubPageModuleVisibility = DataHubPageModuleVisibilityClass
HierarchyModuleParams = HierarchyModuleParamsClass
LinkModuleParams = LinkModuleParamsClass
PageModuleScope = PageModuleScopeClass
RichTextModuleParams = RichTextModuleParamsClass

# fmt: on
