{
  "type": "record",
  "Aspect": {
    "name": "costFeatures"
  },
  "name": "CostFeatures",
  "namespace": "com.linkedin.pegasus2avro.metadata.search.features",
  "fields": [
    {
      "type": [
        "null",
        "float"
      ],
      "name": "costLast30Days",
      "default": null,
      "doc": "Cost over last 30 days. The currency type is set with costCurrencyCode."
    },
    {
      "type": [
        "null",
        {
          "type": "enum",
          "name": "CostCurrencyCode",
          "namespace": "com.linkedin.pegasus2avro.metadata.search.features",
          "symbols": [
            "USD"
          ],
          "doc": " Currency code for the cost "
        }
      ],
      "name": "costCurrencyCode",
      "default": null,
      "doc": "The currency code for the given cost."
    }
  ]
}